@@ -398,6 +398,8 @@ void gi_marshalling_tests_array_out_etc (gint first, gint **ints, gint *length,
 void gi_marshalling_tests_array_inout (gint **ints, gint *length);
 void gi_marshalling_tests_array_inout_etc (gint first, gint **ints, gint *length, gint last, gint *sum);
 
+void gi_marshalling_tests_array_in_nonzero_nonlen (gint first, const guint8 *chars);
+
 /* Zero-terminated */
 
 gchar **gi_marshalling_tests_array_zero_terminated_return (void);
@@ -531,6 +533,7 @@ void gi_marshalling_tests_gvalue_in_with_type (GValue *value, GType type);
 void gi_marshalling_tests_gvalue_in_enum (GValue *value);
 
 void gi_marshalling_tests_gvalue_out (GValue **value);
+void gi_marshalling_tests_gvalue_out_caller_allocates (GValue *value);
 
 void gi_marshalling_tests_gvalue_inout (GValue **value);
 
@@ -731,6 +734,12 @@ struct _GIMarshallingTestsObjectClass
      */
     void  (* vfunc_multiple_out_parameters) (GIMarshallingTestsObject *self, gfloat *a, gfloat *b);
 
+    /**
+     * GIMarshallingTestsObjectClass::vfunc_caller_allocated_out_parameter:
+     * @a: (out):
+     */
+    void  (* vfunc_caller_allocated_out_parameter) (GIMarshallingTestsObject *self, GValue *a);
+
     /**
      * GIMarshallingTestsObjectClass::vfunc_return_value_and_one_out_parameter:
      * @a: (out):
@@ -777,6 +786,7 @@ void gi_marshalling_tests_object_method_with_default_implementation (GIMarshalli
 glong gi_marshalling_tests_object_vfunc_return_value_only (GIMarshallingTestsObject *self);
 void gi_marshalling_tests_object_vfunc_one_out_parameter (GIMarshallingTestsObject *self, gfloat *a);
 void gi_marshalling_tests_object_vfunc_multiple_out_parameters (GIMarshallingTestsObject *self, gfloat *a, gfloat *b);
+void gi_marshalling_tests_object_vfunc_caller_allocated_out_parameter (GIMarshallingTestsObject *self, GValue *a);
 glong gi_marshalling_tests_object_vfunc_return_value_and_one_out_parameter (GIMarshallingTestsObject *self, glong *a);
 glong gi_marshalling_tests_object_vfunc_return_value_and_multiple_out_parameters (GIMarshallingTestsObject *self, glong *a, glong *b);
 gboolean gi_marshalling_tests_object_vfunc_meth_with_error (GIMarshallingTestsObject *object, gint x, GError **error);
@@ -994,6 +1004,8 @@ struct _GIMarshallingTestsPropertiesObject {
     guint64 some_uint64;
     gfloat some_float;
     gdouble some_double;
+    gchar **some_strv;
+    GIMarshallingTestsBoxedStruct* some_boxed_struct; 
 };
 
 struct _GIMarshallingTestsPropertiesObjectClass {
