@@ -3,6 +3,7 @@
 #include <stdlib.h>
 #include <glib-object.h>
 #include <gobject/gvaluecollector.h>
+#include <cairo-gobject.h>
 
 #include "regress.h"
 
@@ -216,12 +217,12 @@ regress_test_closure_one_arg (GClosure *closure, int arg)
 /**
  * regress_test_closure_variant:
  * @closure: GClosure which takes one GVariant and returns a GVariant
- * @arg: (transfer none): a GVariant passed as argument to @closure
+ * @arg: (allow-none) (transfer none): a GVariant passed as argument to @closure
  *
  * Return value: (transfer full): the return value of @closure
  */
 GVariant*
-regress_test_closure_variant (GClosure *closure, const GVariant* arg)
+regress_test_closure_variant (GClosure *closure, GVariant* arg)
 {
   GValue return_value = {0, };
   GValue arguments[1] = {{0,} };
@@ -238,6 +239,8 @@ regress_test_closure_variant (GClosure *closure, const GVariant* arg)
                     NULL);
 
   ret = g_value_get_variant (&return_value);
+  if (ret != NULL)
+    g_variant_ref (ret);
 
   g_value_unset (&return_value);
   g_value_unset (&arguments[0]);
@@ -284,14 +287,18 @@ regress_test_value_return(int i)
 
 /**
  * regress_test_cairo_context_full_return:
+ *
  * Returns: (transfer full):
  */
 cairo_t *
 regress_test_cairo_context_full_return (void)
 {
   cairo_surface_t *surface;
+  cairo_t *cr;
   surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, 10, 10);
-  return cairo_create (surface);
+  cr = cairo_create (surface);
+  cairo_surface_destroy (surface);
+  return cr;
 }
 
 /**
@@ -311,6 +318,7 @@ regress_test_cairo_context_none_in (cairo_t *context)
 
 /**
  * regress_test_cairo_surface_none_return:
+ *
  * Returns: (transfer none):
  */
 cairo_surface_t *
@@ -327,6 +335,7 @@ regress_test_cairo_surface_none_return (void)
 
 /**
  * regress_test_cairo_surface_full_return:
+ *
  * Returns: (transfer full):
  */
 cairo_surface_t *
@@ -429,6 +438,7 @@ static const char utf8_nonconst[] = "nonconst \xe2\x99\xa5 utf8";
 
 /**
  * regress_test_utf8_const_return:
+ *
  * Return value: UTF-8 string
  */
 const char *
@@ -709,7 +719,7 @@ regress_test_strv_in (char **arr)
  * @types: (array length=n_types): List of types
  *
  * Return value: (transfer full): string representation of provided types
- * */
+ */
 char *
 regress_test_array_gtype_in (int n_types, GType *types)
 {
@@ -808,6 +818,7 @@ regress_test_array_fixed_size_int_out (int **ints)
 
 /**
  * regress_test_array_fixed_size_int_return:
+ *
  * Returns: (array fixed-size=5) (transfer full): a list of 5 integers ranging from 0 to 4
  */
 int *
@@ -839,6 +850,7 @@ regress_test_strv_out_c (void)
 /**
  * regress_test_array_int_full_out:
  * @len: length of the returned array.
+ *
  * Returns: (array length=len) (transfer full): a new array of integers.
  */
 int *
@@ -855,6 +867,7 @@ regress_test_array_int_full_out(int *len)
 /**
  * regress_test_array_int_none_out:
  * @len: length of the returned array.
+ *
  * Returns: (array length=len) (transfer none): a static array of integers.
  */
 int *
@@ -1256,6 +1269,14 @@ regress_test_ghash_gvalue_return (void)
   g_value_set_boxed(value, string_array);
   g_hash_table_insert(hash, g_strdup("strings"), value);
 
+  value = g_value_new(REGRESS_TEST_TYPE_FLAGS);
+  g_value_set_flags(value, REGRESS_TEST_FLAG1 | REGRESS_TEST_FLAG3);
+  g_hash_table_insert(hash, g_strdup("flags"), value);
+
+  value = g_value_new(regress_test_enum_get_type());
+  g_value_set_enum(value, REGRESS_TEST_VALUE2);
+  g_hash_table_insert(hash, g_strdup("enum"), value);
+
   return hash;
 }
 
@@ -1295,6 +1316,16 @@ regress_test_ghash_gvalue_in (GHashTable *hash)
   g_assert(strings != NULL);
   for (i = 0; string_array[i] != NULL; i++)
     g_assert(strcmp(strings[i], string_array[i]) == 0);
+
+  value = g_hash_table_lookup(hash, "flags");
+  g_assert(value != NULL);
+  g_assert(G_VALUE_HOLDS_FLAGS(value));
+  g_assert(g_value_get_flags(value) == (REGRESS_TEST_FLAG1 | REGRESS_TEST_FLAG3));
+
+  value = g_hash_table_lookup(hash, "enum");
+  g_assert(value != NULL);
+  g_assert(G_VALUE_HOLDS_ENUM(value));
+  g_assert(g_value_get_enum(value) == REGRESS_TEST_VALUE2);
 }
 
 /**
@@ -1794,6 +1825,38 @@ G_DEFINE_BOXED_TYPE(RegressTestBoxedB,
                     regress_test_boxed_b_copy,
                     regress_test_boxed_b_free);
 
+RegressTestBoxedC *
+regress_test_boxed_c_new (void)
+{
+  RegressTestBoxedC *boxed;
+
+  boxed = g_slice_new (RegressTestBoxedC);
+  boxed->refcount = 1;
+  boxed->another_thing = 42; /* what else */
+
+  return boxed;
+}
+
+static RegressTestBoxedC *
+regress_test_boxed_c_ref (RegressTestBoxedC *boxed)
+{
+  g_atomic_int_inc (&boxed->refcount);
+  return boxed;
+}
+
+static void
+regress_test_boxed_c_unref (RegressTestBoxedC *boxed)
+{
+  if (g_atomic_int_dec_and_test (&boxed->refcount)) {
+    g_slice_free (RegressTestBoxedC, boxed);
+  }
+}
+
+G_DEFINE_BOXED_TYPE(RegressTestBoxedC,
+                    regress_test_boxed_c,
+                    regress_test_boxed_c_ref,
+                    regress_test_boxed_c_unref);
+
 G_DEFINE_TYPE(RegressTestObj, regress_test_obj, G_TYPE_OBJECT);
 
 enum
@@ -1807,7 +1870,8 @@ enum
   PROP_TEST_OBJ_INT,
   PROP_TEST_OBJ_FLOAT,
   PROP_TEST_OBJ_DOUBLE,
-  PROP_TEST_OBJ_STRING
+  PROP_TEST_OBJ_STRING,
+  PROP_TEST_OBJ_GTYPE
 };
 
 static void
@@ -1867,6 +1931,10 @@ regress_test_obj_set_property (GObject      *object,
       self->string = g_value_dup_string (value);
       break;
 
+    case PROP_TEST_OBJ_GTYPE:
+      self->gtype = g_value_get_gtype (value);
+      break;
+
     default:
       /* We don't have any other property... */
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -1920,6 +1988,10 @@ regress_test_obj_get_property (GObject    *object,
       g_value_set_string (value, self->string);
       break;
 
+    case PROP_TEST_OBJ_GTYPE:
+      g_value_set_gtype (value, self->gtype);
+      break;
+
     default:
       /* We don't have any other property... */
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -1960,6 +2032,7 @@ enum {
   REGRESS_TEST_OBJ_SIGNAL_SIG_WITH_HASH_PROP,
   REGRESS_TEST_OBJ_SIGNAL_SIG_WITH_STRV,
   REGRESS_TEST_OBJ_SIGNAL_SIG_WITH_OBJ,
+  REGRESS_TEST_OBJ_SIGNAL_SIG_WITH_FOREIGN_STRUCT,
   REGRESS_TEST_OBJ_SIGNAL_FIRST,
   REGRESS_TEST_OBJ_SIGNAL_CLEANUP,
   REGRESS_TEST_OBJ_SIGNAL_ALL,
@@ -2079,6 +2152,23 @@ regress_test_obj_class_init (RegressTestObjClass *klass)
 		  1,
 		  G_TYPE_OBJECT);
 
+   /**
+   * RegressTestObj::sig-with-foreign-struct:
+   * @self: an object
+   * @cr: (transfer none): A cairo context.
+   */
+  regress_test_obj_signals[REGRESS_TEST_OBJ_SIGNAL_SIG_WITH_FOREIGN_STRUCT] =
+    g_signal_new ("sig-with-foreign-struct",
+		  G_TYPE_FROM_CLASS (gobject_class),
+		  G_SIGNAL_RUN_LAST,
+		  0,
+		  NULL,
+		  NULL,
+                  NULL,
+		  G_TYPE_NONE,
+		  1,
+		  CAIRO_GOBJECT_TYPE_CONTEXT);
+
   regress_test_obj_signals[REGRESS_TEST_OBJ_SIGNAL_FIRST] =
     g_signal_new ("first",
 		  G_TYPE_FROM_CLASS (gobject_class),
@@ -2249,6 +2339,18 @@ regress_test_obj_class_init (RegressTestObjClass *klass)
                                    pspec);
 
 
+  /**
+   * TestObj:gtype:
+   */
+  pspec = g_param_spec_gtype ("gtype",
+                              "GType property",
+                              "A GType property",
+                              G_TYPE_NONE,
+                              G_PARAM_READWRITE);
+  g_object_class_install_property (gobject_class,
+                                   PROP_TEST_OBJ_GTYPE,
+                                   pspec);
+
   klass->matrix = regress_test_obj_default_matrix;
 }
 
@@ -2258,6 +2360,7 @@ regress_test_obj_init (RegressTestObj *obj)
   obj->bare = NULL;
   obj->boxed = NULL;
   obj->hash_table = NULL;
+  obj->gtype = G_TYPE_INVALID;
 }
 
 /**
@@ -2312,6 +2415,14 @@ regress_test_obj_emit_sig_with_obj (RegressTestObj *obj)
     g_object_unref (obj_param);
 }
 
+void
+regress_test_obj_emit_sig_with_foreign_struct (RegressTestObj *obj)
+{
+  cairo_t *cr = regress_test_cairo_context_full_return ();
+  g_signal_emit_by_name (obj, "sig-with-foreign-struct", cr);
+  cairo_destroy (cr);
+}
+
 int
 regress_test_obj_instance_method (RegressTestObj *obj)
 {
@@ -2603,7 +2714,7 @@ regress_test_obj_null_out (RegressTestObj **obj)
 void
 regress_test_array_fixed_out_objects (RegressTestObj ***objs)
 {
-    RegressTestObj **values = g_new(gpointer, 2);
+    RegressTestObj **values = (RegressTestObj**)g_new(gpointer, 2);
 
     values[0] = regress_constructor();
     values[1] = regress_constructor();
@@ -3603,3 +3714,17 @@ regress_test_struct_fixed_array_frob (RegressTestStructFixedArray *str)
   for (i = 0; i < G_N_ELEMENTS(str->array); i++)
     str->array[i] = 42 + i;
 }
+
+/**
+ * regress_has_parameter_named_attrs:
+ * @foo: some int
+ * @attributes: (type guint32) (array fixed-size=32): list of attributes
+ *
+ * This test case mirrors GnomeKeyringPasswordSchema from
+ * libgnome-keyring.
+ */
+void
+regress_has_parameter_named_attrs (int        foo,
+                                   gpointer   attributes)
+{
+}
