@@ -102,6 +102,8 @@ void regress_test_gslist_null_out(GSList **out_list);
 const GHashTable *regress_test_ghash_null_return (void);
 const GHashTable *regress_test_ghash_nothing_return (void);
 GHashTable *regress_test_ghash_nothing_return2 (void);
+GHashTable *regress_test_ghash_gvalue_return (void);
+void regress_test_ghash_gvalue_in (GHashTable *hash);
 GHashTable *regress_test_ghash_container_return (void);
 GHashTable *regress_test_ghash_everything_return (void);
 void regress_test_ghash_null_in (const GHashTable *in);
@@ -120,7 +122,7 @@ GPtrArray *regress_test_garray_full_return (void);
 /* closure */
 int regress_test_closure (GClosure *closure);
 int regress_test_closure_one_arg (GClosure *closure, int arg);
-GVariant *regress_test_closure_variant (GClosure *closure, const GVariant* arg);
+GVariant *regress_test_closure_variant (GClosure *closure, GVariant* arg);
 
 /* value */
 int regress_test_int_value_arg(const GValue *v);
@@ -326,6 +328,17 @@ GType regress_test_boxed_b_get_type (void);
 RegressTestBoxedB *regress_test_boxed_b_new (gint8 some_int8, glong some_long);
 RegressTestBoxedB *regress_test_boxed_b_copy (RegressTestBoxedB *boxed);
 
+typedef struct _RegressTestBoxedC RegressTestBoxedC;
+
+struct _RegressTestBoxedC
+{
+  guint refcount;
+  guint another_thing;
+};
+
+GType regress_test_boxed_c_get_type (void);
+RegressTestBoxedC *regress_test_boxed_c_new (void);
+
 /* gobject */
 #define REGRESS_TEST_TYPE_OBJ              (regress_test_obj_get_type ())
 #define REGRESS_TEST_OBJECT(object)        (G_TYPE_CHECK_INSTANCE_CAST ((object), REGRESS_TEST_TYPE_OBJ, RegressTestObj))
@@ -351,6 +364,7 @@ struct _RegressTestObj
   float some_float;
   double some_double;
   char* string;
+  GType gtype;
 };
 
 struct _RegressTestObjClass
@@ -360,7 +374,7 @@ struct _RegressTestObjClass
   int (*matrix) (RegressTestObj *obj, const char *somestr);
 
   /**
-   * RegressTestObjClass::allow_none_vfunc
+   * RegressTestObjClass::allow_none_vfunc:
    * @two: (allow-none): Another object
    */
   void (*allow_none_vfunc) (RegressTestObj *obj, RegressTestObj *two);
@@ -379,6 +393,7 @@ RegressTestObj*   regress_constructor (void);
 RegressTestObj*   regress_test_obj_new_from_file (const char *x, GError **error);
 void       regress_test_obj_set_bare (RegressTestObj *obj, GObject *bare);
 void       regress_test_obj_emit_sig_with_obj (RegressTestObj *obj);
+void       regress_test_obj_emit_sig_with_foreign_struct (RegressTestObj *obj);
 int        regress_test_obj_instance_method (RegressTestObj *obj);
 double     regress_test_obj_static_method (int x);
 void       regress_forced_method (RegressTestObj *obj);
@@ -781,4 +796,7 @@ void regress_test_struct_fixed_array_frob (RegressTestStructFixedArray *str);
 	"POWERSHARE,PRODIGY,TLX,X400,GIF,CGM,WMF,BMP,MET,PMB,DIB,PICT,TIFF," \
 	"PDF,PS,JPEG,QTIME,MPEG,MPEG2,AVI,WAVE,AIFF,PCM,X509,PGP"
 
+void regress_has_parameter_named_attrs (int        foo,
+                                        gpointer   attributes);
+
 #endif /* __GITESTTYPES_H__ */
